/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package library;

import java.io.File;
import java.io.IOException;
import java.nio.file.FileVisitResult;
import java.nio.file.FileVisitor;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.ArrayList;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import javafx.concurrent.Service;
import javafx.concurrent.Task;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;
import org.apache.log4j.Logger;
import org.jtp.vlcfxmediaplayer.util.Video;
import org.xml.sax.SAXException;
import uk.co.caprica.vlcj.component.DirectMediaPlayerComponent;
import uk.co.caprica.vlcj.player.MediaPlayer;
import uk.co.caprica.vlcj.player.direct.BufferFormat;
import uk.co.caprica.vlcj.player.direct.BufferFormatCallback;
import uk.co.caprica.vlcj.player.direct.format.RV32BufferFormat;

/**
 *
 * @author Dub
 */
public class Library {

    private static final Logger log = Logger.getLogger(Library.class);
    private LibraryDocBuilder docBuilder;
    private LibraryDocParser docParser;
    private ArrayList<LibraryItem> videos = new ArrayList<>();
    private ArrayList<File> files = new ArrayList<>();
    private double thumbWidth = 150;
    private double thumbHeight = thumbWidth / 1.78;
    private final String libFilePath = System.getProperty("user.dir") + "/library";
    private File libDir;
    private final String libFileName = "library.xml";
    private File libFile;
    private String thumbDir = libFilePath + "/thumbs";
    private File thumbFile;

    public Library() {
        try {
            docBuilder = new LibraryDocBuilder();
            docParser = new LibraryDocParser();
            loadLibFile();
        } catch (ParserConfigurationException | IOException ex) {
            log.fatal(ex);
        }

    }

    private void loadLibFile() throws IOException {
        libDir = new File(libFilePath);
        libFile = new File(libFilePath + "/" + libFileName);
        if (!libDir.exists()) {
            if (libDir.mkdir()) {
                log.info("Creating Library Dir");
                if (!libFile.exists()) {
                    try {
                        scanForVideos();
                    } catch (Exception e) {
                        log.fatal(e);
                    }
                }
            }
        } else {
            try {
                videos = docParser.parse(libFile);
                
            } catch (SAXException ex) {
                java.util.logging.Logger.getLogger(Library.class.getName()).log(Level.SEVERE, null, ex);
            } catch (XPathExpressionException ex) {
                java.util.logging.Logger.getLogger(Library.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public ArrayList<LibraryItem> getVideos() {
        return videos;
    }

    public void scanForVideos() {
        new ScanService().start();
    }

    private class ScanService extends Service<ArrayList<File>> {

        @Override
        protected Task<ArrayList<File>> createTask() {
            return new Task<ArrayList<File>>() {
                @Override
                protected ArrayList<File> call() throws Exception {
                    try {
                        Files.walkFileTree(new File("E:\\Movies").toPath(), new FileVisitor<Path>() {
                            @Override
                            public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs) throws IOException {
                                return FileVisitResult.CONTINUE;
                            }

                            @Override
                            public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
                                if (file.toFile().getAbsolutePath().endsWith(".avi") || file.toFile().getAbsolutePath().endsWith(".mp4") || file.toFile().getAbsolutePath().endsWith(".mkv")) {
                                    files.add(file.toFile());
                                }
                                //System.out.println(file.getFileName());
                                return FileVisitResult.CONTINUE;
                            }

                            @Override
                            public FileVisitResult visitFileFailed(Path file, IOException exc) throws IOException {
                                return FileVisitResult.CONTINUE;
                            }

                            @Override
                            public FileVisitResult postVisitDirectory(Path dir, IOException exc) throws IOException {
                                return FileVisitResult.CONTINUE;
                            }
                        });
                    } catch (IOException ex) {
                        log.fatal(ex);
                    }
                    return files;
                }

                @Override
                protected void succeeded() {
                    super.succeeded();
                        log.info("DONE SCANNING FOR FILES.");
                }
            };
        }
    }


    private String timeFormat(long time) {
        long t = time;//mediaLength - time;
        if (t >= 3600000) {
            return String.format("%02d:%02d:%02d",
                    TimeUnit.MILLISECONDS.toHours(t),
                    TimeUnit.MILLISECONDS.toMinutes(t) - TimeUnit.HOURS.toMinutes(TimeUnit.MILLISECONDS.toHours(t)),
                    TimeUnit.MILLISECONDS.toSeconds(t) - TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(t)));
        } else {
            return String.format("%02d:%02d",
                    TimeUnit.MILLISECONDS.toMinutes(t) - TimeUnit.HOURS.toMinutes(TimeUnit.MILLISECONDS.toHours(t)),
                    TimeUnit.MILLISECONDS.toSeconds(t) - TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(t)));
        }
    }
}
