/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package library;

import java.io.File;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.scene.image.Image;
import org.apache.log4j.Logger;
import uk.co.caprica.vlcj.filter.AudioFileFilter;
import uk.co.caprica.vlcj.filter.MediaFileFilter;
import uk.co.caprica.vlcj.filter.VideoFileFilter;
import uk.co.caprica.vlcj.player.MediaPlayerFactory;
import uk.co.caprica.vlcj.player.headless.HeadlessMediaPlayer;

/**
 *
 * @author Dub
 */
public class LibraryItem {
    private static final Logger logger = Logger.getLogger(LibraryItem.class);
    
    private MediaPlayerFactory factory;
    private HeadlessMediaPlayer player;
    
    private static final String thumbPath = System.getProperty("user.dir") + "thumbs";
    private String thumbFile = thumbPath+ "/" + player.getMediaMeta().getTitle() + ".png";
    
    private File source;
    public File getSource(){return source;}
    
    private SimpleStringProperty idProperty;
    public String getId(){return idProperty.get();}
    public void setId(String s){idProperty.set(s);}
    public StringProperty idProperty(){return idProperty;}
    
    private SimpleStringProperty mrlProperty;
    public String getMrl(){return mrlProperty.get();}
    public void setMrl(String s){mrlProperty.set(s);}
    public StringProperty mrlProperty(){return mrlProperty;}
    
    private SimpleStringProperty extensionProperty;
    public String getExtension(){return extensionProperty.get();}
    public void setExtension(String s){extensionProperty.set(s);}
    public StringProperty extensionProperty(){return extensionProperty;}
    
    private SimpleStringProperty typeProperty;
    public String getType(){return typeProperty.get();}
    public void setType(String s){typeProperty.set(s);}
    public StringProperty typeProperty(){return typeProperty;}
    
    private SimpleObjectProperty<Image> thumbnailProperty;
    public Image getThumbnailImage(){return thumbnailProperty.get();}
    public void setThumbnailImage(Image i){thumbnailProperty.set(i);}
    public ObjectProperty<Image> thumbnailProperty(){return thumbnailProperty;}
    
    private static final String[] playerOptions = new String[]{
        "--no-audio",
        "--snapshot-format={png}"
    };
    
    //for loading from xml
    public LibraryItem(String id, String mrl, String extension, String type, Image thumbnail) {
        this.idProperty = new SimpleStringProperty(id);
        this.mrlProperty = new SimpleStringProperty(mrl);
        this.typeProperty = new SimpleStringProperty(type);
        this.extensionProperty = new SimpleStringProperty(extension);
        this.thumbnailProperty = new SimpleObjectProperty<>(thumbnail);
    }   
    
    // for loading from FileScanner
    public LibraryItem(File f){
        this.source = f;
        this.factory = new MediaPlayerFactory();
        this.player = factory.newHeadlessMediaPlayer();
        this.player.prepareMedia(source.getAbsolutePath(), playerOptions);
        
        runFileCheck(source);
    }
    
    private void runFileCheck(File f){        
        if(MediaFileFilter.INSTANCE.accept(f)){
            if(VideoFileFilter.INSTANCE.accept(f)){
                logger.info("MediaFile is Video");
                if(!checkForThumbnail(new File(thumbFile))){                    
                    player.parseMedia();
                    prepareSnapshot(new File(thumbFile));
                    
                    factory.release();
                    player = null;
                    factory = null;
                    
                }else{
                    
                }
            }
            if(AudioFileFilter.INSTANCE.accept(f)){
                logger.info("MediaFile is Audio");
            }
        }        
    }
    private boolean checkForThumbnail(File f){
        logger.info("Checking for thumbnail for: " + f.getName());
        if(f.exists()){
            logger.info("Thumbnail found: " + f.getName());
            return true;
        }else{        
            return false;
        }
    }
    
    private void prepareSnapshot(File f){
        logger.info("Preparing to save a thumbnail");
        player.setPosition(0.72f);
        
        if(!player.saveSnapshot(f)){
            logger.error("Unable to save Thumbnail");
        }else{
            logger.info("Thumbnail saved");
        }
    }
    
}
