/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.jtp.vlcfxmediaplayer.components;

import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javafx.animation.KeyFrame;
import javafx.animation.Timeline;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.collections.ObservableMap;
import javafx.concurrent.Service;
import javafx.concurrent.Task;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.scene.image.ImageView;
import javafx.scene.layout.HBox;
import javafx.scene.layout.StackPane;
import javafx.util.Duration;
import org.jtp.vlcfxmediaplayer.util.Video;

/**
 * FXML Controller class
 *
 * @author Dub
 */
public class SearchBox extends StackPane implements Initializable {
    private ObservableList<Video> mainList;
    private ObservableList<Video> newList;
    private ObservableMap<String,Video> mainMap;
    
    public SearchBox(ObservableList<Video> list){
        this.mainList = list;
        loadFXML();
    }
    public SearchBox(ObservableMap<String,Video> map){
        this.mainMap = map;
        loadFXML();
    }
    
    @FXML
    private HBox searchBox;
    @FXML
    private TextField searchText;
    @FXML
    private Button searchButton;
    @FXML
    private ImageView searchGraphic;
    
    private SimpleBooleanProperty resultsLoadedProperty;

    /**
     * Initializes the controller class.
     */
    @Override
    public void initialize(URL url, ResourceBundle rb) {
        // TODO
        resultsLoadedProperty = new SimpleBooleanProperty(false);
        resultsLoadedProperty.addListener(new ChangeListener<Boolean>() {
            @Override
            public void changed(ObservableValue<? extends Boolean> ov, Boolean t, Boolean t1) {
                if(t1){
                    Timeline timer = new Timeline();
                    timer.getKeyFrames().add(new KeyFrame(Duration.seconds(1), new EventHandler<ActionEvent>() {
                        @Override
                        public void handle(ActionEvent t) {
                            resultsLoadedProperty.set(false);
                        }
                    }));
                    timer.playFromStart();
                }
            }
        });
        loadButtons();
    }        
    private void loadFXML(){
        try {
            FXMLLoader loader = new FXMLLoader(getClass().getResource("/fxml/controls/SearchBox.fxml"));
            loader.setController(this);
            loader.setRoot(this);
            loader.load();
        } catch (IOException ex) {
            Logger.getLogger(SearchBox.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    private void loadButtons(){
        searchButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent t) {
                if(searchText.getText() != null){
                    searchByTitle(searchText.getText());                   
                }
            }
        });
    }
    private void addActions(){}
    
    public void searchByTitle(String name){
        SearchByTitleService s = new SearchByTitleService(name, mainList);
        s.start();        
    }

    public ObservableList<Video> getNewList() {
        return newList;
    }

    public SimpleBooleanProperty getResultsLoadedProperty() {
        return resultsLoadedProperty;
    }
    
    
    
    private class SearchByTitleService extends Service<ObservableList<Video>>{
        
        private ObservableList<Video> list;
        private String searchFor;
        
        public SearchByTitleService(String searchFor, ObservableList<Video> list){
            this.searchFor = searchFor;
            this.list = list;
        }
        @Override
        protected Task<ObservableList<Video>> createTask() {
            return new Task<ObservableList<Video>>() {

                @Override
                protected ObservableList<Video> call() throws Exception {
                    ObservableList<Video> newList = FXCollections.observableArrayList();
                    for(Video v : list){
                        if(v.getName().startsWith(searchFor)){
                            newList.add(v);
                        }
                    }
                    return newList;
                }
            };
        }

        @Override
        protected void succeeded() {
            super.succeeded();            
            newList = getValue();
            resultsLoadedProperty.set(true);
        }
        
    }
}
